/*
 * cobalt_keys - read cobalt front panel buttons to control Kodi
 *
 * Copyright (c) 2015, Gianpaolo Del Matto, www.phunsites.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*  
 *   include pro trinket library
 *   
 *   for some reason, which I had no time to resolve, this behaves very differently with my setup.
 *   according to official examples, only the single "include <ProTrinketKeyBoard.h>" is required.
 *   
 *   however, if I included only the <ProTrinketKeyBoard.h> file, I would always end up with linker errors.
 *   only by adding the full path includes for <cmdline_defs.h>, <ProTrinketKeyboard.h>
 *   and <usbconfig.h>, the compiler would actually be happy with me.
 */
#include </Users/gianpaolo/Documents/Arduino/libraries/Pro_Trinket_USB_Keyboard_Library-master/cmdline_defs.h>
#include </Users/gianpaolo/Documents/Arduino/libraries/Pro_Trinket_USB_Keyboard_Library-master/ProTrinketKeyboard.h>
#include </Users/gianpaolo/Documents/Arduino/libraries/Pro_Trinket_USB_Keyboard_Library-master/usbconfig.h>
#include <ProTrinketKeyboard.h>


/* key definitions */
#define PIN_ARROW_LEFT A0
#define PIN_ARROW_RIGHT A1
#define PIN_ARROW_UP A2
#define PIN_ARROW_DOWN A3
#define PIN_ENTER A4
#define PIN_BACKSPACE A5

/* LED definition */
#define PIN_LED 13

/* arrays to track our states */
int state_KEY_ARROW_LEFT = 1;
int state_KEY_ARROW_RIGHT = 1;
int state_KEY_ARROW_UP = 1;
int state_KEY_ARROW_DOWN = 1;
int state_KEY_ENTER = 1;
int state_KEY_BACKSPACE = 1;

/* init */
void setup() {
      // enable LED pin
      pinMode(PIN_LED, OUTPUT);
      digitalWrite(PIN_LED, LOW);
  
      // button pins as inputs
      pinMode(PIN_ARROW_RIGHT, INPUT);
      pinMode(PIN_ARROW_LEFT, INPUT);
      pinMode(PIN_ARROW_UP, INPUT);
      pinMode(PIN_ARROW_DOWN, INPUT);
      pinMode(PIN_ENTER, INPUT);
      pinMode(PIN_BACKSPACE, INPUT);

      // enable internal pullups
      digitalWrite(PIN_ARROW_RIGHT, HIGH);
      digitalWrite(PIN_ARROW_LEFT, HIGH);
      digitalWrite(PIN_ARROW_UP, HIGH);
      digitalWrite(PIN_ARROW_DOWN, HIGH);
      digitalWrite(PIN_ENTER, HIGH);
      digitalWrite(PIN_BACKSPACE, HIGH);
     
      // start USB
      TrinketKeyboard.begin();
}

/* main loop */
void loop() {
      // always reset the indicator LED, switch it off and make a short sleep
      digitalWrite(PIN_LED, LOW);
      delay(5); // stay below 10 ms or the pseudo-keyboard may get disconnected, see below
      
      // the poll function must be called at least once every 10 ms or cause a keystroke
      // if it is not, then the computer may think that the device has stopped working, and give errors
      TrinketKeyboard.poll();

      // read all inputs at once so even concurrent presses could be detected this way,
      // though I propably wont ever need it
      //
      int r_KEY_ARROW_RIGHT = digitalRead(PIN_ARROW_RIGHT);
      int r_KEY_ARROW_LEFT = digitalRead(PIN_ARROW_LEFT);
      int r_KEY_ARROW_UP = digitalRead(PIN_ARROW_UP);
      int r_KEY_ARROW_DOWN = digitalRead(PIN_ARROW_DOWN);
      int r_KEY_ENTER = digitalRead(PIN_ENTER);
      int r_KEY_BACKSPACE = digitalRead(PIN_BACKSPACE);


      // handle ARROW RIGHT key
      // by checking inverse state, I enforce an absolute debounce,
      // which means, that only one keypress is detected, no matter
      // how long the key is actually pressed.
      // ok, this also prevents things like "key repeat after nn ms",
      // but actually I don't want this: I want to press the keys
      // on the front panel repeatedly. As it turned out for me,
      // the haptic feedback in Kodi seemed more tactile
      // to me than without it.
      //
      if(r_KEY_ARROW_RIGHT != state_KEY_ARROW_RIGHT) {
        
        // right, we send a key press only if the input reads LOW
        // also, I'm quickly flashing the LED on the trinket,
        // that's actually only as a debugging aid to indicate, that
        // "something" was received.
        //
        if(r_KEY_ARROW_RIGHT == LOW) {
          digitalWrite(PIN_LED, HIGH);
          TrinketKeyboard.pressKey(0, KEYCODE_ARROW_RIGHT);
        }

        // save the current key state to compare key states on next
        // iteration again. we even do so when no actual processing
        // happened (INPUT did NOT read LOW)
        //
        state_KEY_ARROW_RIGHT = r_KEY_ARROW_RIGHT;
      }

      // next sections are identical to the one above, they follow the same logic.
      // yes, I know, I did not respect the DRY (don't repeat yourself) pragma ;-)
      // I should propably implement a bitwise-ANDed matrix to make this more lean.

      // handle ARROW LEFT key
      if(r_KEY_ARROW_LEFT != state_KEY_ARROW_LEFT) {
        if(r_KEY_ARROW_LEFT == LOW) {
          digitalWrite(PIN_LED, HIGH);
          TrinketKeyboard.pressKey(0, KEYCODE_ARROW_LEFT);
        }
        state_KEY_ARROW_LEFT = r_KEY_ARROW_LEFT;
      }

      // handle ARROW UP key
      if(r_KEY_ARROW_UP != state_KEY_ARROW_UP) {
        if(r_KEY_ARROW_UP == LOW) {
          digitalWrite(PIN_LED, HIGH);
          TrinketKeyboard.pressKey(0, KEYCODE_ARROW_UP);
        }
        state_KEY_ARROW_UP = r_KEY_ARROW_UP;
      }

      // handle ARROW_DOWN key
      if(r_KEY_ARROW_DOWN != state_KEY_ARROW_DOWN) {
        if(r_KEY_ARROW_DOWN == LOW) {
          digitalWrite(PIN_LED, HIGH);
          TrinketKeyboard.pressKey(0, KEYCODE_ARROW_DOWN);
        }
        state_KEY_ARROW_DOWN = r_KEY_ARROW_DOWN;
      }

      // handle ENTER key
      if(r_KEY_ENTER != state_KEY_ENTER) {
        if(r_KEY_ENTER == LOW) {
          digitalWrite(PIN_LED, HIGH);
          TrinketKeyboard.pressKey(0, KEYCODE_ENTER);
        }
        state_KEY_ENTER = r_KEY_ENTER;
      }


      // handle BACKSPACE key
      if(r_KEY_BACKSPACE != state_KEY_BACKSPACE) {
        if(r_KEY_BACKSPACE == LOW) { 
          digitalWrite(PIN_LED, HIGH);
          TrinketKeyboard.pressKey(0, KEYCODE_BACKSPACE);
        }
        state_KEY_BACKSPACE = r_KEY_BACKSPACE;
      }

      // reset pressed key
      TrinketKeyboard.pressKey(0, 0);
}
